#!/usr/bin/python3
# DpkgControl -*- mode: python; coding: utf-8 -*-
#
# This module implements control file parsing.
#
# Copyright (c) 2001 Adam Heath <doogie@debian.org>
#
# DpkgParagraph is a low-level class, that reads/parses a single paragraph
# from a file object.
#
# DpkgControl uses DpkgParagraph in a loop, pulling out the value of a
# defined key(package), and using that as a key in it's internal
# dictionary.
#
# DpkgSourceControl grabs the first paragraph from the file object, stores
# it in object.source, then passes control to DpkgControl.load, to parse
# the rest of the file.
#
# To test this, pass it a filetype char, a filename, then, optionally,
# the key to a paragraph to display, and if a fourth arg is given, only
# show that field.

# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from .DpkgDatalist import *
from .SignedFile import *

class DpkgParagraph(DpkgOrderedDatalist):
    trueFieldCasing = {}

    def load( self, f ):
        """Paragraph data from a file object"""
        key = None
        while True:
            line = f.readline()
            if not line:
                return
            # skip blank lines until we reach a paragraph
            if line == '\n':
                if not self:
                    continue
                else:
                    return
            if line[ 0 ] != ' ':
                ( truekey, value ) = line.split( ":", 1 )
                key = truekey.lower()
                self.trueFieldCasing[ key ] = truekey
                self[ key ] = value.strip()
            else:
                self[ key ] += "\n%s" % line.strip()

    def _storeField( self, f, value, lead = " " ):
        value = "\n".join(list(map( lambda v, lead = lead: lead + v if v else "", value.splitlines() )))
        f.write( "%s\n" % value )

    def _store( self, f ):
        """Write our paragraph data to a file object"""
        for ( key, value ) in list(self.items()):
            truekey = self.trueFieldCasing[ key ]
            f.write( "%s:" % truekey )
            self._storeField( f, value )

class DpkgControl(DpkgParagraph):
    def load( self, f, source = False ):
        while True:
            para = DpkgParagraph()
            para.load( f )
            if not para:
                break
            if "source" not in para:
                self[ para[ "package" ] ] = para
            elif source:
                self[ "source" ] = para

    def _store( self, f ):
        """Write our control data to a file object"""
        keys = list(self.keys())
        while keys:
            self[ keys.pop( 0 ) ]._store( f )
            if keys:
                f.write( "\n" )

class DpkgSourceControl(DpkgControl):
    def load( self, f ):
        f = SignedFile( f )
        DpkgControl.load( self, f, source = True )

if __name__ == "__main__":
    import sys
    types = { 'p': DpkgParagraph, 'c': DpkgControl, 's': DpkgSourceControl }
    type = sys.argv[ 1 ]
    if type not in types:
        print( "Unknown type `%s'!" % type )
        sys.exit( 1 )
    file = open( sys.argv[ 2 ], "r" )
    data = types[ type ]()
    data.load( file )
    if len( sys.argv ) > 3:
        rargs = sys.argv[ 3: ]
        if type != 'p':
            data = data[ rargs.pop( 0 ) ]
        if rargs:
            data._storeField( sys.stdout, data[ rargs[ 0 ].lower() ], "" )
        else:
            data._store( sys.stdout )
    else:
        data._store( sys.stdout )

# vim:ts=4:sw=4:et:



# tweet -*- mode: python; coding: utf-8 -*-

# Simple tweet support for mini-dinstall.

# Copyright (c) 2010 Christopher R. Gabriel <cgabriel@truelite.it>

# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import logging
import urllib.request

def send(tweet_body, tweet_server, tweet_user, tweet_password):
    """Send tweet; on error, log and continue."""
    logger = logging.getLogger("mini-dinstall")
    post_url = None
    auth_realm = None
    if tweet_server == 'identica':
        post_url = 'http://identi.ca/api/statuses/update.json'
        auth_realm = 'Identi.ca API'
    if tweet_server == 'twitter':
        post_url = 'http://api.twitter.com/1/statuses/update.json'
        auth_realm = 'Twitter API'

    if not post_url:
        logger.exception("Unknown tweet site")
    if not (tweet_user and tweet_password):
        logger.exception("Missing username or password for tweeting")

    auth_handler = urllib.request.HTTPBasicAuthHandler()
    auth_handler.add_password(realm=auth_realm,
                              uri=post_url,
                              user=tweet_user,
                              passwd=tweet_password)
    m_http_opener = urllib.request.build_opener(auth_handler)

    req = urllib.request.Request(post_url)
    req.add_data("status=%s" % tweet_body)
    try:
        handle = m_http_opener.open(req)
        result = handle.read()
        logger.info("Tweet sent to %s (%s)" % (tweet_server, tweet_user))
    except Exception as e:
        logger.exception("Error sending tweet to %s ('%s') via %s: %s: %s" % (tweet_server, tweet_body, tweet_user, type(e), e.args))



# mail -*- mode: python; coding: utf-8 -*-

# Simple mail support for mini-dinstall.

# Copyright (c) 2008 Stephan Sürken <absurd@debian.org>

# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import smtplib
import email.mime.text
import email.utils
import logging

def send(smtp_server, smtp_from, smtp_to, body, subject="mini-dinstall mail notice"):
    """Send email; on error, log and continue."""
    logger = logging.getLogger("mini-dinstall")

    # Create a mime body
    mime_body = email.mime.text.MIMEText(body, 'plain', 'utf-8')
    mime_body['Subject'] = subject
    mime_body['From'] = smtp_from
    mime_body['To'] = smtp_to
    mime_body['Date'] = email.utils.formatdate(localtime=True)
    mime_body['Message-ID'] = email.utils.make_msgid()
    mime_body.add_header('X-Mini-Dinstall', 'YES')

    try:
        # Send via SMTP server
        smtp = smtplib.SMTP(smtp_server)
        smtp.sendmail(smtp_from, [smtp_to], mime_body.as_string())
        logger.info("Mail sent to %s (%s)" % (smtp_to, subject))
    except Exception as e:
        logger.exception("Error sending mail to %s ('%s') via %s: %s: %s", smtp_to, subject, smtp_server, type(e), e.args)



# ChangeFile -*- mode: python; coding: utf-8 -*-

# A class which represents a Debian change file.

# Copyright (c) 2002 Colin Walters <walters@gnu.org>

# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os, re, stat
import logging
from .DpkgControl import *
from .SignedFile import *
from . import misc

class ChangeFileException(Exception):
    def __init__(self, value):
        self._value = value
    def __str__(self):
        return repr(self._value)

class ChangeFile(DpkgParagraph):
    md5_re = r'^(?P<hashsum>[0-9a-f]{32})[ \t]+(?P<size>\d+)[ \t]+(?P<section>[-/a-zA-Z0-9]+)[ \t]+(?P<priority>[-a-zA-Z0-9]+)[ \t]+(?P<file>[0-9a-zA-Z][-+:.,=~0-9a-zA-Z_]+)$'
    sha1_re = r'^(?P<hashsum>[0-9a-f]{40})[ \t]+(?P<size>\d+)[ \t]+(?P<file>[0-9a-zA-Z][-+:.,=~0-9a-zA-Z_]+)$'
    sha256_re = r'^(?P<hashsum>[0-9a-f]{64})[ \t]+(?P<size>\d+)[ \t]+(?P<file>[0-9a-zA-Z][-+:.,=~0-9a-zA-Z_]+)$'

    def __init__(self):
        DpkgParagraph.__init__(self)
        self._logger = logging.getLogger("mini-dinstall")
        self._file = ''

    def load_from_file(self, filename):
        self._file = filename
        f = SignedFile(open(self._file))
        self.load(f)
        f.close()

    def getFiles(self):
        return self._get_checksum_from_changes()['md5']

    def _get_checksum_from_changes(self):
        """ extract checksums and size from changes file """
        output = {}
        hashes = {
            'md5': ['files', re.compile(self.md5_re)],
            'sha1': ['checksums-sha1', re.compile(self.sha1_re)],
            'sha256': ['checksums-sha256', re.compile(self.sha256_re)]
        }

        if 'files' not in self:
            return []

        for (hash, (field, regex)) in list(hashes.items()):
            if field not in self:
                self._logger.warn("Can't find %s checksum in changes file '%s'" % (hash, os.path.basename(self._file)))
                continue
            output[hash] = []
            for line in self[field].splitlines():
                if not line:
                    continue
                match = regex.match(line)
                if not match:
                    raise ChangeFileException("Couldn't parse file entry \"%s\" in %s field of .changes" % (line, self.trueFieldCasing[field]))
                output[hash].append([match.group('hashsum'), match.group('size'), match.group('file')])
        return output

    def verify(self, sourcedir):
        """ verify size and hash values from changes file """
        checksum = self._get_checksum_from_changes()
        for (hash, value) in list(checksum.items()):
            for (hashsum, size, filename) in value:
                self._verify_file_integrity(os.path.join(sourcedir, filename), int(size), hash, hashsum)

    def _verify_file_integrity(self, filename, expected_size, hash, expected_hashsum):
        """ check uploaded file integrity """
        self._logger.debug('Checking integrity of %s' % filename)
        try:
            statbuf = os.stat(filename)
            if not stat.S_ISREG(statbuf[stat.ST_MODE]):
                raise ChangeFileException("%s is not a regular file" % filename)
            size = statbuf[stat.ST_SIZE]
        except OSError as e:
            raise ChangeFileException("Can't stat %s: %s" % (filename, e.strerror))
        if size != expected_size:
            raise ChangeFileException("File size for %s does not match that specified in .dsc" % filename)
        if misc.get_file_sum(self, hash, filename) != expected_hashsum:
            raise ChangeFileException("%ssum for %s does not match that specified in .dsc" % (hash, filename))
        self._logger.debug('Verified %ssum %s and size %s for %s' % (hash, expected_hashsum, expected_size, filename))

# vim:ts=4:sw=4:et:



# Dnotify -*- mode: python; coding: utf-8 -*-

# A simple FAM-like beast in Python

# Copyright (c) 2002 Colin Walters <walters@gnu.org>

# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os, stat, threading, queue, time
import logging
from . import misc

class DnotifyException(Exception):
    def __init__(self, value):
        self._value = value
    def __str__(self):
        return repr(self._value)

class DirectoryNotifierFactory:
    def create(self, dirs, use_dnotify=False, poll_time=30, logger=None, cancel_event=None):
        if use_dnotify and os.access('/usr/bin/dnotify', os.X_OK):
            if logger:
                logger.debug("Using dnotify directory notifier")
            return DnotifyDirectoryNotifier(dirs, logger)
        else:
            if logger:
                logger.debug("Using mtime-polling directory notifier")
            return MtimeDirectoryNotifier(dirs, poll_time, logger, cancel_event=cancel_event)

class DnotifyNullLoggingFilter(logging.Filter):
    def filter(self, record):
        return 0

class DirectoryNotifier:
    def __init__(self, dirs, logger, cancel_event=None):
        self._cwd = os.getcwd()
        self._dirs = dirs
        if not cancel_event:
            self._cancel_event = threading.Event()
        else:
            self._cancel_event = cancel_event
        if not logger:
            self._logger = logging.getLogger("Dnotify")
            self._logger.addFilter(DnotifyNullLoggingFilter())
        else:
            self._logger = logger

    def cancelled(self):
        return self._cancel_event.isSet()

class DirectoryNotifierAsyncWrapper(threading.Thread):
    def __init__(self, dnotify, queue, logger=None, name=None):
        if name:
            threading.Thread.__init__(self, name=name)
        else:
            threading.Thread.__init__(self)
        self._eventqueue = queue
        self._dnotify = dnotify
        if not logger:
            self._logger = logging.getLogger("Dnotify")
            self._logger.addFilter(DnotifyNullLoggingFilter())
        else:
            self._logger = logger

    def cancel(self):
        self._cancel_event.set()

    def run(self):
        self._logger.info('Created new thread (%s) for async directory notification' % self.getName())
        while not self._dnotify.cancelled():
            dir = self._dnotify.poll()
            self._eventqueue.put(dir)
        self._logger.info('Caught cancel event; async dnotify thread exiting')

class MtimeDirectoryNotifier(DirectoryNotifier):
    def __init__(self, dirs, poll_time, logger, cancel_event=None):
        DirectoryNotifier.__init__(self, dirs, logger, cancel_event=cancel_event)
        self._changed = []
        self._dirmap = {}
        self._polltime = poll_time
        for dir in dirs:
            self._dirmap[dir] = os.stat(os.path.join(self._cwd, dir))[stat.ST_MTIME]

    def poll(self, timeout=None):
        timeout_time = None
        if timeout:
            timeout_time = time.time() + timeout
        while not self._changed:
            if timeout_time and time.time() > timeout_time:
                return None
            self._logger.debug('Polling...')
            for dir in list(self._dirmap.keys()):
                oldtime = self._dirmap[dir]
                mtime = os.stat(os.path.join(self._cwd, dir))[stat.ST_MTIME]
                if oldtime < mtime:
                    self._logger.debug('Directory "%s" has changed' % dir)
                    self._changed.append(dir)
                self._dirmap[dir] = mtime
            if not self._changed:
                for x in range(self._polltime):
                    if self._cancel_event.isSet():
                        return None
                    time.sleep(1)
        ret = self._changed.pop(0)
        return ret

class DnotifyDirectoryNotifier(DirectoryNotifier):
    def __init__(self, dirs, logger):
        DirectoryNotifier.__init__(self, dirs, logger)
        self._queue = queue.Queue()
        dnotify = DnotifyThread(self._queue, self._dirs, self._logger)
        dnotify.start()

    def poll(self, timeout=None):
        # delete duplicates
        i = self._queue.qsize()
        self._logger.debug('Queue size: %d' % i)
        set = {}
        while i:
            dir = self._queue_get(timeout)
            if not dir:
                # We shouldn't have to do this; no one else is reading
                # from the queue.  But we do it just to be safe.
                for key in list(set.keys()):
                    self._queue.put(key)
                return None
            set[dir] = 1
            i -= 1
        for key in list(set.keys()):
            self._queue.put(key)
        i = self._queue.qsize()
        self._logger.debug('Queue size (after duplicate filter): %d' % i)
        return self._queue_get(timeout)

    def _queue_get(self, timeout):
        if not timeout:
            return self._queue.get()
        timeout_time = time.time() + timeout
        while True:
            try:
                self._queue.get(0)
            except queue.Empty:
                if time.time() > timeout_time:
                    return None
                else:
                    time.sleep(15)

class DnotifyThread(threading.Thread):
    def __init__(self, queue, dirs, logger):
        threading.Thread.__init__(self)
        self._queue = queue
        self._dirs = dirs
        self._logger = logger

    def run(self):
        self._logger.debug('Starting dnotify reading thread')
        (infd, outfd) = os.pipe()
        pid = os.fork()
        if pid == 0:
            os.close(infd)
            misc.dup2(outfd, 1)
            args = ['dnotify', '-m', '-c', '-d', '-a', '-r'] + list(self._dirs) + ['-e', 'printf', '"{}\\0"']
            os.execv('/usr/bin/dnotify', args)
            os.exit(1)

        os.close(outfd)
        stdout = os.fdopen(infd)
        c = 'x'
        while c:
            curline = ''
            c = stdout.read(1)
            while c and c != '\0':
                curline += c
                c = stdout.read(1)
            if not c:
                break
            self._logger.debug('Directory "%s" changed' % curline)
            self._queue.put(curline)
        (pid, status) = os.waitpid(pid, 0)
        if not status:
            ecode = 0
        else:
            ecode = os.WEXITSTATUS(status)
        raise DnotifyException("dnotify exited with code %s" % ecode)

# vim:ts=4:sw=4:et:



#!/usr/bin/python
#
#  setup.py - [ install script for mini-dinstall ]
#  Copyright (c) 2007 Christoph Goehre <christoph.goehre@gmx.de>
#
#  This file is part of the mini-dinstall package.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.
#

from distutils.core import setup

setup(name = 'mini-dinstall',
      author = 'Colin Walters',
      author_email = 'walters@debian.org',
      maintainer = 'Christoph Goehre',
      maintainer_email = 'christoph.goehre@gmx.de',
      scripts = [ 'mini-dinstall' ],
      packages = [ 'minidinstall' ],
      url = 'http://alioth.debian.org/projects/mini-dinstall/',
)



# misc -*- mode: python; coding: utf-8 -*-

# misc tools for mini-dinstall

# Copyright (c) 2004 Thomas Viehmann <tv@beamnet.de>

# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os, errno, time, re, hashlib

def dup2(fd, fd2):
    # dup2 with EBUSY retries (cf. dup2(2) and Debian bug #265513)
    success = False
    tries = 0
    while not success:
        try:
            os.dup2(fd, fd2)
            success = True
        except OSError as e:
            if e.errno != errno.EBUSY or tries >= 3:
                raise
            # wait 0-2 seconds before next try
            time.sleep(tries)
            tries += 1

def format_changes(L):
    """ remove changelog header and all lines with only a dot """
    dotmatch = re.compile('^\.$')
    L1 = []

    for x in L[3:]:
        L1.append(dotmatch.sub('', x))

    return "\n".join(L1)

def get_file_sum(self, type, filename):
    """ generate hash sums for file """
    sum = getattr(hashlib, type)()
    self._logger.debug("Generate %s (python-internal) for %s" % (type, filename))
    f = open(filename, 'rb')
    buf = f.read(8192)
    while buf:
        sum.update(buf)
        buf = f.read(8192)
    return sum.hexdigest()
